#include "CConstants.h"

#include "CParts2.h"
#include "CParts.h"

#include "Compat.h"
#include "CCursor.h"
#include "CUtils.h"

#include <stdarg.h>
#include <va_list.h>

#define Begin Creator->MySetPort();  RGBColor bc = Creator->GetBackColor(); RGBBackColor(&bc);
#define End ::BackColor(whiteColor); Creator->RestoreWorld();

CCommandButton::CCommandButton(short MenuID, short MenuItem, short theIcon,  \
	Rect theRect, CWindowBase* owner): cicnButton(theIcon, theRect, owner)
{
	mID = MenuID;
	mItem = MenuItem;
	myCommand = (MenuID << 16) + (MenuItem);
}

void CCommandButton::DoAction()
{
	Creator->GetCreator()->DoMenuCommand(myCommand);
}

void CCommandButton::DoIdle()
{
	MenuHandle myMenu = Creator->GetCreator()->GetMenu(mID)->GetMenu();
	short Enabled = ((**myMenu).enableFlags & (1 << mItem));
	if (Enabled != Actif) {
		if (Enabled) Enable();
		else Disable();
		}
}

#pragma mark -

CPict::CPict(short pictID, Rect theRect, CWindowBase* owner, short Type, CResFileBase* theFile):CRectPart(theRect, owner, Type)
{
	rFile = theFile;
	if (rFile) {
		rFile->UseThisFile();
		myPicture = GetPicture(pictID);
		rFile->UseHomeFile();
		}
	else
		myPicture = GetPicture(pictID);
}

void CPict::DrawPart()
{
	DrawPicture(myPicture, &mPartRect);
}

void CPict::ChangePicture(short newID)
{
	if (rFile) {
		rFile->UseThisFile();
		myPicture = GetPicture(newID);
		rFile->UseHomeFile();
		}
	else
		myPicture = GetPicture(newID);
	InvalPart();
}

void CPict::_Enable()
{
	Creator->MySetPort();
	DrawPart();
	Creator->RestoreWorld();
}

void CPict::_Disable()
{
	Creator->MySetPort();
	DrawPart();
	Creator->RestoreWorld();
}

#pragma mark -

CTextPart::CTextPart(string text, Rect theRect, CWindowBase* owner, short ActType):CRectPart(theRect, owner, ActType)
{
	myText = text;
	Font = kFontIDGeneva;
	Size = 9;
	Face = 0;
	Justify = teJustLeft;
}

void CTextPart::SetText(string text)
{
	myText = text;
	_UpDateText(true);
}

void CTextPart::SetFont(short theFont)
{
	Font = theFont;
	_UpDateText(false);
}

void CTextPart::SetSize(short theSize)
{
	Size = theSize;
	_UpDateText(false);
}

void CTextPart::SetFace(short theFace)
{
	Face = theFace;
	_UpDateText(false);
}

void CTextPart::SetJustify(short theJustify)
{
	Justify = theJustify;
	_UpDateText(true);
}

void CTextPart::WriteData(char *Msg,...)
{
	va_list args;
	va_start(args, Msg);
//	Str255 MyS;
//	char *s = (char*)(&MyS) + 1;
	static char s[250];
	short l = vsprintf(s, Msg, args);
//	MyS[0] = l;
	
	SetText(s);
	
	va_end(args);
}

void CTextPart::ReadData(char *Msg,...)
{
	va_list args;
	va_start(args, Msg);

	string Texte = GetText();
	__vsscanf(Texte.c_str(), Msg, args);

	va_end(args);
}

#pragma mark -

CStaticText::CStaticText(string text, Rect theRect, CWindowBase* owner, short ActType):CTextPart(text, theRect, owner, ActType)
{
	mActive = true;
	withFrame = false;
}

void CStaticText::DrawPart()
{
	if (withFrame) {
		if (!mActive) RGBForeColor(&FondGris2);
		Rect theFrame = mPartRect;
		InsetRect(&theFrame, -3, -3);
		BackColor(whiteColor);
		EraseRect(&theFrame);
		FrameRect(&theFrame);
		ForeColor(blackColor);
		}
	TextFont(Font);
	TextSize(Size);
	TextFace(Face);
	if (withFrame)
		BackColor(whiteColor);
	else
		RGBBackColor(&GrisClair);
	if (mActive) {
		ForeColor(blackColor);
		TETextBox((void*)myText.c_str(),myText.length(), &mPartRect, Justify);
		PenNormal();
		}
	else {
		RGBForeColor(&FondGris2);
		TETextBox((void*)myText.c_str(),myText.length(), &mPartRect, Justify);
		ForeColor(blackColor);
		}
	BackColor(whiteColor);
}

void CStaticText::ShowFrame(short theFrame)
{
	withFrame = theFrame;
	Creator->MySetPort();
	DrawPart();
	Creator->RestoreWorld();
}

void CStaticText::_Enable()
{
	Creator->MySetPort();
	mActive = true;
	DrawPart();
	Creator->RestoreWorld();
}

void CStaticText::_Disable()
{
	Creator->MySetPort();
	mActive = false;
	DrawPart();
	Creator->RestoreWorld();
}

void CStaticText::_UpDateText(short)
{
	Creator->MySetPort();
	CStaticText::DrawPart();
	Creator->RestoreWorld();
}

#pragma mark -

CEditText::CEditText(string text, Rect theRect, CWindowBase* owner, short ActType):CTextPart(text, theRect, owner, ActType)
{
	isActif = true;
	ActualRect = theRect;
	InsetRect(&ActualRect, -3, -3);
	owner->MySetPort();
	Rect viewRect = theRect;
	InsetRect(&viewRect, 0, -1);
	myTEHandle = TENew(&viewRect, &theRect);
	
	CEditText::_UpDateText(true);
	owner->RestoreWorld();
}

CEditText::~CEditText()
{
	TEDispose(myTEHandle);
}

void CEditText::DrawFocus()
{
	Rect FocRect = ActualRect;
	InsetRect(&FocRect, -3, -3);
	PenSize(2,2);
	RGBColor bc = Creator->GetBackColor();
	if ((isActif) && (isFocused))
		RGBForeColor(&FondGris2);
	else
		RGBForeColor(&bc);
	FrameRect(&FocRect);
	ForeColor(blackColor);
	PenSize(1,1);
}

void CEditText::DrawPart()
{
	BackColor(whiteColor);
	EraseRect(&ActualRect);
	if (!isActif)
		RGBForeColor(&FondGris2);
	TEUpdate(&mPartRect, myTEHandle);
	FrameRect(&ActualRect);
/*	Rect myRect = ActualRect;
	InsetRect(&myRect, -1, -1);
	if (isActif) {
		OmbrageRect(myRect);
		}
	else {
		RGBForeColor(&Creator->GetBackColor());
		FrameRect(&myRect);
		}*/
	DrawFocus();
	PenNormal();
}

short CEditText::DoClick(EventRecord* theEvent)
{
	TEClick(theEvent->where, false, myTEHandle);
	return CTextPart::DoClick(theEvent);
}

void CEditText::DoIdle()
{
	if (isFocused)
		TEIdle(myTEHandle);
}

void CEditText::GiveFocus(short FromTab)
{
	isFocused = true;
	(**myTEHandle).selStart = 0;
	(**myTEHandle).selEnd = (FromTab)?32000:0;
	TEActivate(myTEHandle);
	DrawFocus();
	Creator->GetCreator()->PostEvent(new CEvent(kInternalEvent, kSetupMenus));
}

void CEditText::TakeFocus()
{
	isFocused = false;
	TEDeactivate(myTEHandle);
	DrawFocus();
	Creator->GetCreator()->PostEvent(new CEvent(kInternalEvent, kSetupMenus));
}

void CEditText::EnterPart()
{
	CCursor::SetCursor(kInsertCursor);
}

string CEditText::GetText()
{
	string Result = "";
	Result.append((char*)(*((**myTEHandle).hText)), (**myTEHandle).teLength);
	return Result;
}

short CEditText::DoKey(EventRecord* theEvent)
{
	if (isFocused) {
		short Char  = theEvent->message & charCodeMask;
		TEKey(Char, myTEHandle);
		return CTextPart::DoKey(theEvent);
		}
	else
		return false;
}

short CEditText::DoEnable(short MenuID, short MenuItem)
{
 	switch (MenuID) {
 		case mEdit:
 			switch (MenuItem) {
 				case iCopy:
 				case iCut:
 				case iClear:
 					return ((isFocused) && (isActif) && ((**myTEHandle).selStart != (**myTEHandle).selEnd));
 				case iPaste:
					return ((isFocused) && (isActif));
 				case iSelAll:
					return ((isFocused) && (isActif));
 	 			}
 	 	}
 	 return false;
}

short CEditText::DoCommand(short MenuID, short MenuItem)
{
	if ((isFocused) && (isActif)) {
	 	switch (MenuID) {
	 		case mEdit:
	 			switch (MenuItem) {
	 				case iCopy:
	 					TECopy(myTEHandle);
	 					return true;
	 				case iCut:
	 					TECut(myTEHandle);
						Creator->DoAction(ActionType, 0);
	 					return true;
	 				case iClear:
	 					TEDelete(myTEHandle);
						Creator->DoAction(ActionType, 0);
	 					return true;
	 				case iPaste:
	 					TEPaste(myTEHandle);
						Creator->DoAction(ActionType, 0);
	 					return true;
	 				case iSelAll:
	 					TESetSelect(0,32000, myTEHandle);
	 					return true;
	 	 			}
	 	 	}
 		 }
 	 return false;
}

void CEditText::SetPartRect(Rect newRect)
{
	CRectPart::SetPartRect(newRect);
	
	ActualRect = newRect;
	InsetRect(&ActualRect, -3, -3);
	Rect viewRect = newRect;
	OffsetRect(&viewRect, 0, -2);
	
	(**myTEHandle).destRect = ActualRect;
	(**myTEHandle).viewRect = viewRect;
}

void CEditText::_Enable()
{
	Creator->MySetPort();
	isActif = true;
	if (isFocused)
		TEActivate(myTEHandle);
	(**myTEHandle).txFace = Face;
	DrawPart();
	Creator->RestoreWorld();
}

void CEditText::_Disable()
{
	Creator->MySetPort();
	isActif = false;
	TEDeactivate(myTEHandle);
	DrawPart();
	Creator->RestoreWorld();
}

void CEditText::_UpDateText(short UpDateText)
{
	Creator->MySetPort();
	if (UpDateText)
		TESetText(myText.c_str(), myText.length(), myTEHandle);

	(**myTEHandle).txFont = Font;
	(**myTEHandle).txSize = Size;
	(**myTEHandle).txFace = Face;

	CEditText::DrawPart();
	Creator->RestoreWorld();
}

void CEditText::InvalPart()
{
	Creator->MySetPort();
	InvalRect(&ActualRect);
	Creator->RestoreWorld();
}

#pragma mark -

CControlPart::CControlPart(short Type, string Title, Rect theRect, CWindowBase* owner, short cMin, short cMax, short ActType): \
	CTextPart("", theRect, owner, ActType)
{
	Str255 Name;
	String2Pas(Title, Name);
	owner->MySetPort();
	RGBColor bc = Creator->GetBackColor();
	RGBBackColor(&bc);
	theControl = NewControl(owner->GetWindow(), &theRect, Name, true, 0, cMin, cMax, Type, 0);
	BackColor(whiteColor);
	owner->RestoreWorld();
	CControlPart::_UpDateText(true);
}

CControlPart::~CControlPart()
{
	if (Creator->GetWindow()) {
		Creator->MySetPort();
		RGBColor bc = Creator->GetBackColor();
		RGBBackColor(&bc);
		DisposeControl(theControl);
		BackColor(whiteColor);
		Creator->RestoreWorld();
		}
}

void CControlPart::SetData(short Part, unsigned long Tag, long Size, void* Data)
{
	SetControlData(theControl, Part, Tag, Size, (Ptr)Data);
}

void CControlPart::DrawPart()
{
	RGBColor bc = Creator->GetBackColor();
	RGBBackColor(&bc);
	Draw1Control(theControl);
	BackColor(whiteColor);
}

short CControlPart::DoClick(EventRecord* theEvent)
{
	ControlHandle myControl;
	short handled = true;
	short Part = FindControl(theEvent->where, Creator->GetWindow(), &myControl);
	if (*myControl == *theControl) {
		RGBColor bc = Creator->GetBackColor();
		RGBBackColor(&bc);
		short Res = TrackControl(theControl, theEvent->where, 0);
		::BackColor(whiteColor);
		if (Res) {
			Creator->DoAction(ActionType, Res);
			return true;
			}
		}
	return false;
}

void CControlPart::SetValue(short val)
{
	Begin
	::SetControlValue(theControl, val);
	End
}

void CControlPart::SetMin(short val)
{
	Begin
	::SetControlMinimum(theControl, val);
	End
}

void CControlPart::SetMax(short val)
{
	Begin
	::SetControlMaximum(theControl, val);
	End
}

short CControlPart::GetValue()
{
	Begin
	short val = ::GetControlValue(theControl);
	End
	return val;
}

void CControlPart::SetHilite(short val)
{
	Begin
	::HiliteControl(theControl, val);
	End
}

void CControlPart::SetPartRect(Rect newRect)
{
	CRectPart::SetPartRect(newRect);
	MoveControl(theControl, newRect.left, newRect.top);
	SizeControl(theControl, newRect.right - newRect.left, newRect.bottom - newRect.top);
}

void CControlPart::_UpDateText(short)
{
	ControlFontStyleRec theStyle;
	theStyle.font = Font;
	theStyle.size = Size;
	theStyle.style = Face;
	theStyle.flags = kControlUseFaceMask + kControlUseFontMask + kControlUseSizeMask + kControlUseBackColorMask;
	Creator->MySetPort();
	RGBColor bc = Creator->GetBackColor();
	RGBBackColor(&bc);
	SetControlFontStyle(theControl, &theStyle);
	Draw1Control(theControl);
	::BackColor(whiteColor);
	Creator->RestoreWorld();
}

void CControlPart::_Enable()
{
	Creator->MySetPort();
	RGBColor bc = Creator->GetBackColor();
	RGBBackColor(&bc);
	ActivateControl(theControl);
	::BackColor(whiteColor);
	Creator->RestoreWorld();
}

void CControlPart::_Disable()
{
	Creator->MySetPort();
	RGBColor bc = Creator->GetBackColor();
	RGBBackColor(&bc);
	DeactivateControl(theControl);
	::BackColor(whiteColor);
	Creator->RestoreWorld();
}

#pragma mark -

CFrame::CFrame(Rect theRect, CWindowBase* owner):CStaticText("", theRect, owner)
{
	mActif = true;
	drawName = false;
	Creator->MySetPort();
	CFrame::DrawPart();
	Creator->RestoreWorld();
}

short CFrame::DoClick(EventRecord*)
{
	return false;
}

void CFrame::_Enable()
{
	mActif = true;
	Creator->MySetPort();
	DrawPart();
	Creator->RestoreWorld();
}

void CFrame::_Disable()
{
	mActif = false;
	Creator->MySetPort();
	DrawPart();
	Creator->RestoreWorld();
}

void CFrame::DrawPart()
{
	ShadowRect(mPartRect, mActif);
}

void CFrame::PutTitle(string title)
{
	SetText(title);
	drawName = true;
	InvalPart();
}

void CFrame::RemoveTitle()
{
	myText = "";
	drawName = false;
	InvalPart();
}

void CFrame::_UpDateText(short)
{
}

