#include "CUtils.h"
#include "CUtilClasses.h"
#include "CBaseClasses.h"
#include "CConstants.h"

void ShadowRect(Rect MonRect, short Actif)
{
	if (Actif) {
		RGBForeColor(&FondGris2);
		MoveTo(MonRect.right-1, MonRect.top);
		LineTo(MonRect.right-1, MonRect.bottom-1);
		LineTo(MonRect.left, MonRect.bottom-1);
		ForeColor(whiteColor);
		LineTo(MonRect.left, MonRect.top);
		LineTo(MonRect.right-1, MonRect.top);
		}
	else {
		RGBForeColor(&FondGris);
		FrameRect(&MonRect);
		}
	ForeColor(blackColor);
}

long24::operator long()
{
	return ((((c1 << 16) + (c2 << 8) + c3)+8388608)%16777216) - 8388608;
}

long24::long24(long y)
{
	unsigned long z = (y+16777216)%16777216;
	c3 = z;
	c2 = z >> 8;
	c1 = z >> 16;
}

int operator==(const SoundCar&Car1, const SoundCar&Car2)
{
	return ((Car1.mSampleRate == Car2.mSampleRate) && \
		(Car1.mSampleSize == Car2.mSampleSize) && \
		(Car1.mNumChans == Car2.mNumChans));
}

int operator==(const SoundSel&Sel1, const SoundSel&Sel2)
{
	return ((Sel1.mSelStart == Sel2.mSelStart) && \
		(Sel1.mSelEnd == Sel2.mSelEnd) && \
		(Sel1.mSelMode == Sel2.mSelMode));
}

Rect ScreenBits()
{
	return (**GetMainDevice()).gdRect;
}

World::World()
{
	theWorld = 0;
	theDevice = 0;
}

World::~World()
{
	theWorld = 0;
	theDevice = 0;
}

void CWorldObj::SaveWorld()
{
	World *myWorld = new World;
	GetGWorld(&(myWorld->theWorld), &(myWorld->theDevice));
	Worlds.Add(myWorld);
}

void CWorldObj::RestoreWorld()
{
	World *aWorld = Worlds.GetLast();
	SetGWorld(aWorld->theWorld, aWorld->theDevice);
	Worlds.DestroyData(aWorld);
}

#pragma mark -

COffScreenWorld::COffScreenWorld(short nBits, Rect theRect, CApplicationBase* Creator)
{
	theWorld = 0;
	theCreator = Creator;
	myPixMap = 0;
	myCTable = 0;
	locked = false;
	_Construct(nBits, theRect);
}

COffScreenWorld::~COffScreenWorld()
{
	if (theWorld) DisposeGWorld(theWorld);
}

void COffScreenWorld::_Construct(short nBits, Rect theRect)
{
	if (locked) ::UnlockPixels(myPixMap);
	if (theWorld) {
		(**theWorld->portPixMap).pmTable = 0;
		DisposeGWorld(theWorld);
		}
	WorldRect = theRect;
	OffsetRect(&WorldRect, -WorldRect.left, -WorldRect.top);
	Depth = nBits;
	SaveWorld();
	short succ = theCreator->ReserveMem((WorldRect.bottom*WorldRect.right*nBits)/8);
	if (succ)
		NewGWorld(&theWorld, nBits, &WorldRect, myCTable, 0, 0);
	else {
		SetRect(&WorldRect,0,0,0,0);
		NewGWorld(&theWorld, nBits, &WorldRect, myCTable, 0, 0);
		theCreator->PostEvent(new CEvent(kCreateError, 0, 2));
		}
	myPixMap = GetGWorldPixMap(theWorld);
	if (locked) ::LockPixels(myPixMap);
	SetGWorld(theWorld, 0);
	EraseRect(&WorldRect);
	RestoreWorld();
}

void COffScreenWorld::ChangePalette(PaletteHandle newPal)
{
	if (!newPal) {
		myCTable = 0;
		(**theWorld->portPixMap).pmTable = 0;
		}
	if (myCTable) {
		Palette2CTab(newPal, myCTable);
//		(**theWorld->portPixMap).pmTable = myCTable;
		}
	else {
		myCTable = (CTabHandle)(NewHandle(16));
		(**myCTable).ctSize = 0;
		Palette2CTab(newPal, myCTable);
		(**theWorld->portPixMap).pmTable = myCTable;
		}
}

void COffScreenWorld::ChangeRect(Rect newRect)
{
	if (!EqualRect(&newRect,&WorldRect))
		_Construct(Depth, newRect);
}

void COffScreenWorld::ChangeDepth(short newDepth)
{
	if (newDepth != Depth)
		_Construct(newDepth, WorldRect);
}

void COffScreenWorld::ChangeWorld(Rect newRect, short newDepth)
{
	if ((!EqualRect(&newRect,&WorldRect)) || (newDepth != Depth))
		_Construct(newDepth, newRect);
}

void COffScreenWorld::CopyTo(CWindowBase*Dest, Rect &srcRect, Rect &dstRect)
{
	myPixMap = GetGWorldPixMap(theWorld);
	if (!locked) ::LockPixels(myPixMap);

	CopyBits((BitMapPtr)*myPixMap, &Dest->GetWindow()->portBits, &srcRect, &dstRect, srcCopy, 0);

	if (!locked) ::UnlockPixels(myPixMap);
}

void COffScreenWorld::CopyTo(COffScreenWorld*Dest, Rect &srcRect, Rect &dstRect)
{
	myPixMap = GetGWorldPixMap(theWorld);
	if (!locked) ::LockPixels(myPixMap);
	PixMapHandle myPixMap2 = GetGWorldPixMap(Dest->theWorld);
	if (!Dest->locked) ::LockPixels(myPixMap2);

	CopyBits((BitMapPtr)*myPixMap, (BitMapPtr)*myPixMap2, &srcRect, &dstRect, srcCopy, 0);

	if (!locked) ::UnlockPixels(myPixMap);
	if (!Dest->locked) ::UnlockPixels(myPixMap2);
}

#pragma mark -

void CHandle::SetSize(long theSize) {
	::SetHandleSize(mHandle, theSize);
	long test = ::GetHandleSize(mHandle);
	if (test != theSize)
		throw MemoryError();
}

void *CHandle::GetDatas(long Pos)
{
	return (void*)(*mHandle + Pos);
}

long CHandle::GetSize() {
	return GetHandleSize(mHandle);
}

void CHandle::SetHandle(Handle newH, short destroyOld) {
	if (destroyOld)
		DisposeHandle(mHandle);
	mHandle = newH;
	}

CHandle::CHandle(long theSize) {
	mHandle = NewHandle(theSize);
	if (!mHandle) throw MemoryError();
}

CHandle::~CHandle() {
	DisposeHandle(mHandle);
}

#pragma mark -

CIOHandle::CIOHandle(): CHandle(0)
{
	theSize = 0;
	theMark = 0;
}

CIOHandle::~CIOHandle()
{
}

void CIOHandle::SetSize(long size)
{
	theSize = size;
	if (theMark > theSize) theMark = theSize;
	CHandle::SetSize(size);
}

void CIOHandle::SetHandle(Handle newH, short destroyOld) {
	CHandle::SetHandle(newH, destroyOld);
	theSize = GetHandleSize(newH);
	}

void CIOHandle::WriteData(void* data, long size)
{
	if (theMark + size > theSize)
		SetSize(theMark + size);
	HLock(mHandle);
	BlockMove(data, (void*)((long)*mHandle + theMark), size);
	HUnlock(mHandle);
	theMark += size;
}

void CIOHandle::ReadData(void* data, long size)
{
	HLock(mHandle);
	BlockMove((Ptr)((long)*mHandle + theMark), data, size);
	HUnlock(mHandle);
	theMark += size;
}

#define DeclareWrite(Type) CIOHandle &CIOHandle::operator<<(Type data) { 	\
	short Err = 0; 																							\
	long Size = sizeof(Type); 																			\
	WriteData((void*)&data, Size); 																\
	return *this; } 
	
CIOHandle &CIOHandle::operator<<(string &data) 
{
	short length = data.length(), Err = 0;
	(*this) << length;
	
	long l = length;
	WriteData((void*)data.c_str(), l);
	return *this;
}
	
DeclareWrite(short)
DeclareWrite(long)
DeclareWrite(double)
DeclareWrite(complex<double>)
DeclareWrite(Boolean)

#define DeclareRead(Type) CIOHandle &CIOHandle::operator>>(Type &data) { 	\
	short Err = 0; 																								\
	long Size = sizeof(Type); 																				\
	ReadData((void*)&data, Size); 																		\
	return *this; }
	
CIOHandle &CIOHandle::operator>>(string &data) 
{
	short length, Err = 0;
	(*this) >> length;
	char *mot = new char[length+1];
	
	long l = length;
	ReadData((void*)mot, l);
	mot[length] = 0;
	data = string(mot);
	delete mot;
	return *this;
}
	
DeclareRead(short) 
DeclareRead(long)
DeclareRead(double)
DeclareRead(complex<double>)
DeclareRead(Boolean) 


CPtr::CPtr(long theSize, short clear)
{
#define Try(SIZE)  if (clear) mData = NewPtrClear(SIZE); else mData = NewPtr(SIZE);

	if (theSize > 0) {
		Try(theSize);
		if (!mData) {
			Handle myHan = NewHandle(theSize);
			if (myHan) DisposeHandle(myHan);
			Try(theSize);
			}
		if (!mData)
			throw MemoryError();
		}
	else {
		short notGood = false;
		theSize = ::MaxBlock() - 400000;
		do {
			try {
				theSize -= 50000;
				Try(theSize);
				if (!mData) {
					Handle myHan = NewHandle(theSize);
					if (myHan) DisposeHandle(myHan);
					Try(theSize);
					}
				if (!mData) 
					throw 1;
				notGood = false;
				}
			catch (...) {
				notGood = true;
				}
			} while (notGood);
		}
}

CMenu::CMenu(MenuHandle monMenu, short oldItem, CResFileBase* owner)
{
	theMenu = monMenu;
	theFile = owner;
	if (oldItem > 0)
		menuID = oldItem;
	else
		menuID = (**theMenu).menuID;
}
