#include "CExternalStuff.h"
#include "CControllers.h"
#include "NewFilter.h"

class CPitchController: public CStandardModalController {
	CApplicationBase* theApplication;
	CResFileBase* theFile;
	
	double factor;
public:
	CPitchController(double, CApplicationBase*, CResFileBase*);
	~CPitchController() {}
	
	virtual void HandleSelect(short);
	virtual void OpenDialog(CWindowBase*);
	virtual void CloseDialog();
	
	double GetPitch() {return factor;}
	};
	
CPitchController::CPitchController(double fact, CApplicationBase* app, CResFileBase* file):CStandardModalController()
{
	theFile = file;
	theApplication = app;
	factor = fact;
}
	
void CPitchController::OpenDialog(CWindowBase* myDialog)
{
	CStandardModalController::OpenDialog(myDialog);

	myDialog->GetTextItem(3)->SetJustify(teJustRight);
	myDialog->GetTextItem(4)->SetJustify(teJustRight);

	myDialog->WriteItem(5,"%1.3lf", factor);
	myDialog->WriteItem(6, "%1.3lf", 12*log(factor)/log(2));
}

void CPitchController::CloseDialog()
{
	factor = theDialog->ReadBox(5);
}

void CPitchController::HandleSelect(short SelectID)
{
	switch (SelectID) {
		case 1:
			if ((factor > 10) || (factor < 0.1)) {
				_exterror(1, theApplication, theFile);
				return;
				}
			break;
		case 5:
			factor = theDialog->ReadBox(5);
			theDialog->WriteItem(6, "%1.3lf", 12*log(factor)/log(2));
			break;
		case 6:
			double tones = theDialog->ReadBox(6);
			factor = exp(tones*log(2)/12);
			theDialog->WriteItem(5, "%1.3lf", factor);
			break;
		}
	CStandardModalController::HandleSelect(SelectID);
}

void ChangePitch(CSoundBase* theSound, CAmadBase* theApplication, CResFileBase* theFile)
{
	CPitchController* theController = new CPitchController(1.0, theApplication, theFile);
	CWindowBase* myBox = theApplication->CreateModalBox(159, true, 0, theController);

	myBox->Run();
	if (theController->WasOK()) {
		double myPitch = theController->GetPitch();
		SoundCar theCar = theSound->GetCar();
		theCar.mNumChans = theSound->GetSel().NChansSel();
		CSoundBase* tempSound = theApplication->CreateSound(0, theCar);
		tempSound->SelectAll();
		tempSound->CopyFrom(theSound);
		SoundCar newCar = theCar;
		newCar.mSampleRate /= myPitch;
		newCar.mFixedRate /= myPitch;
		tempSound->ChangeFormat(newCar);
		tempSound->ChangeCar(theCar);
		tempSound->SelectAll();
		theSound->CopyFrom(tempSound);
		}
	delete myBox;
	delete theController;
}

pascal void main(CSoundBase* theSound, CWindowBase* SoundWindow, CAmadBase* theApplication, CResFileBase* theFile, short theItem, long* myData)
{
	switch (theItem) {
		case kEnableItem:
			*myData = (theSound->GetSel().mSelStart != theSound->GetSel().mSelEnd);
			break;
		case 0:
			ChangePitch(theSound, theApplication, theFile);
			break;
		}
}