#ifndef CBASECLASSES
#define CBASECLASSES

#include "CUtilClasses.h"
#include "CList.h"
#include <string.h>

// Classes defined in this file \\

class AnyObject;
class CSoundBase;
class CApplicationBase;
class CAmadBase;
class CWindowBase;
class CSoundAccess;

//******************//

class CPart;
class CRectPart;
class CTextPart;
class CFilter;
class CTransform;
class CController;
class CMark;
class CMenu;
class CResFileBase;
class CEvent;

#define _delete(win) {win->GetCreator()->PostEvent(new CEvent(1003, (long)(win->GetWindow())));}
#define _exterror(ID, Appl, File) Appl->PostEvent(new CEvent(kExtError, (long)File, ID))

enum {
	kCenter		= 1,
	kAlert		= 2
	};

class AnyObject {
public:
	short ClassType;
	long ObjData;
	AnyObject() {ClassType = ObjData = 0;}
	virtual ~AnyObject() {}
};

class CSoundAccess {
public:
	CSoundAccess() {}
	virtual ~CSoundAccess() {}

	virtual CSample &GetSample(ulong, short) = 0;
	virtual CSample *GetSamplePtr(ulong, short) = 0;
	
	virtual void PositionAccess(ulong) = 0;	
	virtual void ResizeAccess(ulong) = 0;	
	virtual void UpDateAccess() = 0;
	
	virtual ulong GetSize() = 0;
	virtual ulong GetPos() = 0;
	};
	
	
#pragma mark CSoundBase

class CSoundBase {
public:
	virtual ~CSoundBase() {};
	virtual CSoundAccess* GetReadAccess(ulong, ulong) = 0;
	virtual CSoundAccess* GetWriteAccess(ulong, ulong) = 0;
	virtual ulong MaxAccess() = 0;
	
	virtual long BeginSel() = 0;
	virtual long EndSel() = 0;
	virtual short SelMode() = 0;
	virtual void SetSel(long, long, long=0) = 0;
	virtual void SetSel(SoundSel) = 0;
	virtual SoundSel GetSel() = 0;

	virtual short SampleSize() = 0;
	virtual short NBytes() = 0;
	virtual short NChans() = 0;
	virtual long SampleRate() = 0;
	virtual Fixed Taux() = 0;
	virtual long Length() = 0;
	virtual ulong DataLength() = 0;
	virtual void ChangeDataLength(ulong) = 0;
	virtual void SelectAll() = 0;
	virtual short MaxMode() = 0;
	virtual void PutZeros(long, long, short) = 0;
	
	virtual void TryLoadSound() = 0;
	
	virtual void ChangeCar(SoundCar newCar) = 0;
	virtual void ChangeFormat(SoundCar) = 0;
	virtual SoundCar GetCar() = 0;
	virtual void CopyAllFrom(CSoundBase*) = 0;
	
	virtual void ChangeMarks() {}
	virtual CList<CMark> *GetMarks() {return 0;}
	virtual void SetMarks(CList<CMark>*) {}
public:
// Standard Manipulations
	virtual void ChangeLength(ulong) = 0;
	virtual void Clear() = 0;
	virtual void CopyFrom(CSoundBase*) = 0;
	virtual void AddSoundFrom(CSoundBase*, ulong, short) = 0;
	};

class CApplicationBase {
protected:
	virtual short DoEnable(short, short) = 0;
	virtual short GetMark(short, short) = 0;
	virtual void SetMenus() = 0;
	virtual void DoUpdate(WindowPtr) = 0;
	virtual void DoActive(WindowPtr, short) = 0;
	virtual void DoEvent(EventRecord*) = 0;
	virtual void DoIdleWindows() = 0;
	virtual void Exit() = 0;
public:
	virtual void RemovePart(CPart*) = 0;
	virtual short DragRealTime() {return false;}
public:
	virtual short DoCommand(short, short) = 0;
	virtual void DoMenuCommand(long) = 0;
	virtual void FreezeApp(short) = 0;
	virtual short FreezeState() = 0;

	virtual void Run() = 0;
	virtual void DoIdle() = 0;
	
	virtual CMenu *GetMenu(short, CResFileBase* = 0) = 0;
	virtual void PostEvent(CEvent*) = 0;
	virtual short ReserveMem(ulong) = 0;
	
	virtual void AddWindow(CWindowBase*) = 0;
	virtual void RemoveWindow(CWindowBase*) = 0;
	virtual void AddPalette(CWindowBase*) = 0;
	virtual void RemovePalette(CWindowBase*) = 0;
	
	virtual CList<CWindowBase> *GetWindows() = 0;
	virtual CList<CWindowBase> *GetPalettes() = 0;
	
	virtual short WasError() = 0;
	};

#pragma mark CAmadBase

class CAmadBase: virtual public CApplicationBase {
public:
	virtual CSoundBase* CreateSound(ulong, SoundCar) = 0;
	virtual CWindowBase* CreateFilterProcess(CSoundBase*, CFilter*) = 0;
	virtual CWindowBase* CreateTransformProcess(CSoundBase*, CTransform*) = 0;
	virtual CWindowBase* CreateDialog(short, short, CResFileBase*, CController* = 0, short = 1) = 0;
	virtual CWindowBase* CreateModalBox(short, short, CResFileBase*, CController* = 0, short = 1) = 0;
	};

#pragma mark CWindowBase

class CWindowBase: virtual public AnyObject {
public:
// Do not use these functions
	virtual void _AddMother(CWindowBase*) = 0;
	virtual void _RemoveDaughter(CWindowBase*) = 0;
public:
	virtual void Show() = 0;
	virtual void Hide() = 0;
	virtual CApplicationBase *GetCreator() = 0;
	
	virtual void Draw() = 0;
	virtual void DoClick(EventRecord*) = 0;
	virtual short DoKey(EventRecord*) = 0;
	virtual short DoAction(short, long) = 0;
	virtual void DoIdle() = 0;
	virtual void ActiveDesactive(short) = 0;
	virtual short AcceptClose() {return true;}
	virtual Rect GetGrowRect() = 0;
	virtual void ChangeSize(long) {}
	virtual void Select() = 0;
	virtual short IsActive() = 0;
	
	virtual void StopDrawing() = 0;
	virtual void ResumeDrawing() = 0;
	
	virtual void GiveFocus(CPart*, short) = 0;
	virtual void AdvanceFocus(short) = 0;
	
	virtual RGBColor GetBackColor() = 0;
	virtual void AddPart(CPart*) = 0;
	
	virtual short DoCommand(short, short) = 0;
	virtual short DoEnable(short, short) = 0;
	virtual short GetMark(short, short) = 0;
	
	virtual void MySetPort() = 0;
	virtual void SaveWorld() = 0;
	virtual void RestoreWorld() = 0;
	virtual short WantPalette(CWindowBase*) = 0;
	
	virtual void AddDaughter(CWindowBase*) = 0;
	virtual CList<CWindowBase> *GetDaughters() = 0;
	virtual CList<CPart> *GetPartList() = 0;
	virtual WindowPtr GetWindow() = 0;
	
// Available for Dialogs only

	virtual void InstallController(CController*) {}
	virtual void RemoveItems(short, short = -1) {}
	virtual void AppendItems(short, short, short) {}

 	virtual Rect GetItemRect(short) {return *(Rect*)0;}
 	virtual void InstallItem(short, CRectPart*) {};
 	virtual CRectPart* GetItem(short i) {return 0;}
 	virtual CTextPart* GetTextItem(short i) {return 0;}
	
	virtual void SetCtlValue(short, short) {}
	virtual short GetCtlValue(short) {return 0;}
	virtual void SetCtlHilite(short, short) {}
	
	virtual double ReadBox(short) {return 0;}
	virtual long ReadBoxInt(short) {return 0;}
	virtual string ReadBoxString(short) {return "";}
	virtual double ReadBoxFun(short) {return 0;}
	
	virtual void WriteItem(short, char*,...) {}
	virtual void ReadItem(short, char*,...) {}
	
// Available for Modal Boxes only

	virtual void Run() {}
	virtual void CloseBox() {}
};

#endif