#ifndef CLIST
#define CLIST

#include "CError.h"

enum {
	ListRes		= 6002,
	
	IllegalAccess		= 1,
	IndexError		= 2,
	EmptyError		= 3,
	NotFoundError	= 4
};

class ListError: public Error{
public:
	ListError(short ID):Error(ID) {}
	~ListError() {}
	void HandleError() {StandardHandler(ErrorType, ListRes);}
};

#define forlist(List,Element) for (Element = (List).GetFirst();(Element); Element = (List).GetNext())

template<class T> class CLink {
	T* mData;
	CLink<T> *mNext;
public:
	CLink(T *Data);
	CLink(CLink<T> *model);
	~CLink();
	
	T* GetData() {return mData;}
	void SetData(T* newData) {mData = newData;}
	void SetNext(CLink<T> *Next) {mNext = Next;}
	short HasNext() {if (mNext) return true; else return false;}
	CLink<T> *GetNext() {return mNext;}
};

template<class T> CLink<T>::CLink(T *Data)
{
	mData = Data;
	mNext = 0;
}

template<class T> CLink<T>::CLink(CLink<T> *model)
{
	mData = new T(*(model->mData));
	mNext=0;
}

template<class T> CLink<T>::~CLink()
{
	if (mData)
		delete mData;
}

template<class T> class CList {
protected:
	CLink<T> *last;
	CLink<T> *current;
	CLink<T> *save;
	CLink<T> *first;
	long mSize;
public:
	CList(CList<T>*);
	CList() {mSize = 0;last=0;current=0;save = 0;first = 0;}
	~CList();
	
	T *operator[](long);
	T *LastData() {if (last) return last->GetData(); return 0;}
	T *GetFirst() {if (!first) return 0; current = first;return current->GetData();}
	T *GetLast() {if (!last) return 0; current = last;return current->GetData();}
	T *GetNext();
	T *GetCurrent() {if (current) return current->GetData(); return 0;}
	
	void ReplaceData(long, T*);
	
	long GetIndex(T*);
	long GetCurrentIndex(){if (current) return GetIndex(current->GetData()); return -1;};
	void Delete(long);
	void DeleteNext();
	void Remove(long);
	void Remove(T*);
	void Insert(T*, long);
	void Add(T*);
	long GetSize() {return mSize;}
	
	CLink<T> *CurrentLink() {return current;}
	CLink<T> *FirstLink() {return first;}
	CLink<T> *LastLink() {return last;}
	void SetCurrent(CLink<T> *Curr) {current = Curr;}
	void SetCurrent(T* Curr);
	void SetCurrent(long);
	
	void SavePos() {save = current;}
	void RestorePos() {current = save;}
	
	void Destroy();
	
	void DestroySegment(T*,T*);
	void DestroyData(T*);
	void InsertCopySegment(CList<T>*, long);
	void InsertSegment(CList<T>*, long);
	void AppendSegment(CList<T> *);
	void InsertAfterCurrent(CList<T>*);
	void InsertAfterCurrent(T*);
	CList<T> *RemoveSegment(T*,T*);
	CList<T> *DuplicateSegment(T*,T*);
};

template<class T> CList<T>::CList(CList<T> *Orig)
{
	mSize = Orig->mSize;
	if (mSize) {
		save = 0;
		CLink<T> *CurLink = Orig->first;
		
		first = new CLink<T>(CurLink);
		CurLink = CurLink->GetNext();
		current = first;
		CLink<T> *newlink;
		
		while (CurLink) {
			newlink = new CLink<T>(CurLink);
			current->SetNext(newlink);
			current = newlink;
			CurLink = CurLink->GetNext();
			}
		last = current;
		current = first;
		}
	else
		first = last = current = save = 0;
}

template<class T> CList<T>::~CList()
{
	if (mSize == 0) return;
	CLink<T> *myCur;
	CLink<T> *myOld = first;
	while (myOld) {
		myCur = myOld->GetNext();
		myOld->SetData(0);
		delete myOld;
		myOld = myCur;
		}
	first = last = current = 0;
	mSize = 0;
}

template<class T> void CList<T>::Destroy()
{
	if (mSize == 0) return;
	CLink<T> *myCur;
	CLink<T> *myOld = first;
	while (myOld) {
		myCur = myOld->GetNext();
		delete myOld;
		myOld = myCur;
		}
	first = last = current = 0;
	mSize = 0;
}

template<class T> T *CList<T>::GetNext()
{
	if ((current) && (current->HasNext())) {
		current = current->GetNext();
		return current->GetData();
		}
	else {
		current = 0;
		return 0;
		}
}

template<class T> void CList<T>::Add(T *Data)
{
	if (mSize == 0) {
		last = new CLink<T>(Data);
		first = current = last;
		mSize = 1;
		return;
		}
	CLink<T> *mySave = last;
	last = new CLink<T>(Data);
	if (mySave)
		mySave->SetNext(last);
	else
		first = last;
	current = last;
	mSize++;
}

template<class T> void CList<T>::SetCurrent(T *Data)
{
	current = first;
	while ((current->GetData() != Data) && (current)) {
		current = current->GetNext();
		}
	if (!current)
		throw ListError(IndexError);
}

template<class T> void CList<T>::SetCurrent(long Pos)
{
	current = first;
	for (long i=0; i<Pos; i++)
		if (current)
			current = current->GetNext();
		else
			throw ListError(IndexError);
}

template<class T> void CList<T>::Insert(T *Data, long Index)
{
	if (Index<0 || Index > mSize)
		throw ListError(IndexError);
	if ((Index > 0) && (Index < mSize)) {
		current = first;
		for (long i=0; i<Index-1; i++)
			current = current->GetNext();
		CLink<T> *myLink = new CLink<T>(Data);
		myLink->SetNext(current->GetNext());
		current->SetNext(myLink);
		}
	if (Index == mSize) {
		CLink<T> *myLink = new CLink<T>(Data);
		last->SetNext(myLink);
		last = myLink;
		current = last;
		}
	if (Index == 0) {
		CLink<T> *myLink = new CLink<T>(Data);
		myLink->SetNext(first);
		first = myLink;
		if (!mSize)
			last = first;
		}
	mSize++;
}

template<class T> void CList<T>::InsertAfterCurrent(T *Data)
{
	if (!current) throw ListError(IndexError);
	CLink<T> *myLink = new CLink<T>(Data);
	myLink->SetNext(current->GetNext());
	current->SetNext(myLink);
	if (last == current) last = myLink;
	current = myLink;
	mSize++;
}

template<class T> void CList<T>::InsertAfterCurrent(CList<T> *Seg)
{
	if (!current) throw ListError(IndexError);
	Seg->last->SetNext(current->GetNext());
	current->SetNext(Seg->first);
	if (last == current) last = Seg->last;
	current = Seg->last;
	mSize += Seg->mSize;
}

template<class T> void CList<T>::InsertCopySegment(CList<T> *Seg, long Index)
{
	if (Seg->mSize == 0) return;
	if (Index<0 || Index > mSize)
		throw ListError(IndexError);
	CLink<T> *newCurrent = Seg->first;
	CLink<T> *oldLink = new CLink<T>(new T(newCurrent->GetData()));
	CLink<T> *firstLink = oldLink;
	CLink<T> *newLink = oldLink;
	newCurrent = newCurrent->GetNext();
	while (newCurrent) {
		newLink = new CLink<T>(new T(newCurrent->GetData()));
		oldLink->SetNext(newLink);
		oldLink = newLink;
		newCurrent = newCurrent->GetNext();
		};
	
	CLink<T> *current;
	if (Index) {
		current = first;
		for (long i=0; i<Index-1; i++)
			current = current->GetNext();
		newLink->SetNext(current->GetNext());
		current->SetNext(firstLink);
		if (last == current) last = newLink;
		}
	else {
		newLink->SetNext(first);
		first = firstLink;
		if (!mSize)
			last = newLink;
		}
	mSize+=Seg->mSize;
}

template<class T> void CList<T>::InsertSegment(CList<T> *Seg, long Index)
{
	if (Seg->mSize == 0) return;
	if (Index<0 || Index > mSize)
		throw ListError(IndexError);
	
	CLink<T> *current;
	if (Index) {
		current = first;
		for (long i=0; i<Index-1; i++)
			current = current->GetNext();
		Seg->last->SetNext(current->GetNext());
		current->SetNext(Seg->first);
		if (last == current) last = Seg->last;
		}
	else {
		Seg->last->SetNext(first);
		first = Seg->first;
		if (!mSize)
			last = Seg->last;
		}
	mSize+=Seg->mSize;
	Seg->mSize = 0;
	Seg->first = Seg->last = Seg->current = 0;
}

template<class T> void CList<T>::AppendSegment(CList<T> *Seg)
{
	if (Seg->mSize == 0) return;

	if (mSize) {
		last->SetNext(Seg->first);
		last = Seg->last;
		current = last;
		}
	else {
		first = Seg->first;
		last = Seg->last;
		current = last;
		}

	mSize+=Seg->mSize;
	Seg->mSize = 0;
	Seg->first = Seg->last = Seg->current = 0;
}

template<class T> void CList<T>::Remove(long Index)
{
	if (mSize <= 0)
		throw ListError(EmptyError);
	if (Index<0 || Index >= mSize)
		throw ListError(IndexError);
	
	if (Index) {
		current = first;
		for (long i=0; i<Index-1; i++)
			current = current->GetNext();
		if (current->GetNext() == last) {
			last->SetData(0);
			delete last;
			last = current;
			current->SetNext(0);
			}
		else {
			CLink<T> *save = current->GetNext()->GetNext();
			current->SetNext(save);
			current->GetNext()->SetData(0);
			delete current->GetNext();
			}
		}
	else {
		if (current == first) current = 0;
		CLink<T> *mySave = first;
		first = first->GetNext();
		mySave->SetData(0);
		delete mySave;
		if (mSize==1) {
			first = 0;
			last = 0;
			current = 0;
			}
		}
		
	mSize--;
}

template<class T> long CList<T>::GetIndex(T* Ref)
{
	if (mSize <= 0)
		throw ListError(NotFoundError);
	
	short ind = 0;
	current = first;
	while (current) {
		if (current->GetData() == Ref)
			return ind;
		ind++;
		current = current->GetNext();
		}
	
	throw ListError(NotFoundError);
}

/*long CList<Word>::GetIndex(Word* Ref)
{
	if (mSize <= 0)
		throw ListError(NotFoundError);
	
	short ind = 0, print = false;
	current = first;
	while (current) {
		if (print) {
			cout << current->GetData();
			cout.flush();
			}
		if (current->GetData() == Ref)
			return ind;
		ind++;
		current = current->GetNext();
		}
	
	throw ListError(NotFoundError);
}*/

template<class T> void CList<T>::Remove(T* Obsolete)
{
	if (mSize <= 0)
		return;
	
	if (first->GetData() != Obsolete) {
		if (first == last) return;
		CLink<T> *prec = first;
		current = first->GetNext();
		while ((current->GetData() != Obsolete) && (current != last)) {
			prec = current;
			current = current->GetNext();
			}
		if ((current == last) && (current->GetData() != Obsolete))
			return;
		if (current == last) {
			last = prec;
			prec->SetNext(0);
			}
		else
			prec->SetNext(current->GetNext());
		current->SetData(0);
		delete current;
		current = prec;
		}
	else {
		current = first;
		first = first->GetNext();
		current->SetData(0);
		delete current;
		current = 0;
		if (mSize==1) {
			first = 0;
			last = 0;
			}
		}
	mSize--;
}

template<class T> void CList<T>::Delete(long Index)
{
	if (mSize <= 0)
		throw ListError(EmptyError);
	if (Index<0 || Index >= mSize)
		throw ListError(IndexError);
	
	if (Index) {
		current = first;
		for (long i=0; i<Index-1; i++)
			current = current->GetNext();
		if (current->GetNext() == last) {
			delete last;
			last = current;
			}
		else {
			CLink<T> *save = current->GetNext()->GetNext();
			delete current->GetNext();
			current->SetNext(save);
			}
		}
	else {
		current = first->GetNext();
		delete first;
		first = current;
		if (mSize==1) {
			first = 0;
			last = 0;
			current = 0;
			}
		}
		
	mSize--;
}

template<class T> void CList<T>::DeleteNext()
{
	if ((current == last) || (!current))
		throw ListError(IndexError);
	
	if (current->GetNext() == last) {
		delete last;
		last = current;
		}
	else {
		CLink<T> *save = current->GetNext()->GetNext();
		delete current->GetNext();
		current->SetNext(save);
		}
	mSize--;
}

template<class T> T *CList<T>::operator[](long Index)
{
	if (Index<0 || Index > mSize)
		throw ListError(IndexError);
	if (Index == mSize) {
		return 0;
		}
	current = first;
	for (long i=0; i<Index; i++)
		current = current->GetNext();
	return current->GetData();
}

template<class T> void CList<T>::ReplaceData(long Index, T* newData)
{
	if (Index<0 || Index >= mSize)
		throw ListError(IndexError);
	current = first;
	for (long i=0; i<Index; i++)
		current = current->GetNext();
	current->SetData(newData);
}

template<class T> CList<T> *CList<T>::DuplicateSegment(T* firstD, T* lastD)
{
	CList<T> *resultList = new CList<T>;
	CLink<T> *myFirst;
	CLink<T> *newcur;
	newcur = first;
	while (newcur->GetData() != firstD) {
		myFirst = newcur;
		newcur = newcur->GetNext();
		if (!newcur) throw ListError(IndexError);
		}
	resultList->Add(new T(newcur->GetData()));
	while (newcur->GetData() != lastD) {
		newcur = newcur->GetNext();
		resultList->Add(new T(newcur->GetData()));
		if (!newcur) throw ListError(IndexError);
		}
	return resultList;
}

template<class T> CList<T> *CList<T>::RemoveSegment(T* firstD, T* lastD)
{
	CList<T> *resultList = new CList<T>;
	CLink<T> *myLast, *myFirst = 0;
	CLink<T> *newcur = first;
	long nDel = 0;
	while (newcur->GetData() != firstD) {
		myFirst = newcur;
		newcur = newcur->GetNext();
		if (!newcur) throw ListError(IndexError);
		}
	resultList->first = newcur;
	while (newcur->GetData() != lastD) {
		newcur = newcur->GetNext(); nDel++;
		if (!newcur) throw ListError(IndexError);
		}
	nDel++;
	resultList->mSize = nDel;
	mSize -= nDel;
	resultList->last = newcur;
	resultList->current = resultList->first;
	
	current = newcur->GetNext();
	myLast = newcur->GetNext();
	newcur->SetNext(0);
	if (myFirst)
		myFirst->SetNext(myLast);
	else
		first = myLast;
	if (!myLast)
		last = myFirst;
	return resultList;
}

template<class T> void CList<T>::DestroySegment(T* firstD, T* lastD)
{
	CLink<T> *saveL, *myFirst = 0;
	current = first;
	long nDel = 0;
	while (current->GetData() != firstD) {
		myFirst = current;
		current = current->GetNext();
		if (!current) throw ListError(IndexError);
		}
	while (current->GetData() != lastD) {
		saveL = current->GetNext();
		delete current; nDel++;
		current = saveL;
		if (!current) throw ListError(IndexError);
		}
	saveL = current->GetNext();
	delete current; nDel++;
	current = saveL;
	if (myFirst)
		myFirst->SetNext(current);
	else
		first = current;
	if (!current) last = myFirst;
	mSize -= nDel;
}

template<class T> void CList<T>::DestroyData(T* firstD)
{
	if (mSize == 0)
		throw ListError(IndexError);
	if (mSize == 1) {
		if (first->GetData() == firstD) {
			delete first;
			first = last = current = 0;
			mSize = 0;
			}
		else 
			throw ListError(IndexError);
		}
	else {
		CLink<T> *saveL, *myFirst = 0;
		current = first;
		while (current->GetData() != firstD) {
			myFirst = current;
			current = current->GetNext();
			if (!current) throw ListError(IndexError);
			}
		saveL = current->GetNext();
		if (current == first)
			first = saveL;
		if (current == last)
			last = myFirst;
		delete current; 
		current = saveL;
		if (myFirst)
			myFirst->SetNext(current);
		mSize --;
		}
}

#endif