#ifndef CUTILCLASSES
#define CUTILCLASSES

struct SoundCar {
	long mSampleRate;
	short mSampleSize;
	short mNumChans;
	Fixed mFixedRate;
	
	short SampleSize() {return mNumChans * mSampleSize;}
	};
	
typedef unsigned long ulong;

struct SoundSel {
	long mSelStart;
	long mSelEnd;
	long mSelMode;
	
	SoundSel() {mSelStart = mSelEnd = mSelMode = 0;}
	short NChansSel() {short res = 0; for (long i=0; i<32; i++) \
		res += ((mSelMode>>i)%2); return res; }
	long Length() {return mSelEnd - mSelStart;}
	};

int operator==(const SoundCar&, const SoundCar&);
int operator==(const SoundSel&, const SoundSel&);

struct long24 {
	unsigned char c1,c2,c3;
	operator long();
	long24(long=0);
	};

union CSample {
	char m8Bits;
	short m16Bits;
	long24 m24Bits;
	
	void put8(long num) {m8Bits = (num >> 16);}
	void put16(long num) {m16Bits = (num >> 8);}
	void put24(long num) {m24Bits = num;}
	
#define Test if (num > 8388607) num = 8388607; if (num < -8388608) num = -8388608;
	void putTest8(long num) {Test m8Bits = (num >> 16);}
	void putTest16(long num) {Test m16Bits = (num >> 8);}
	void putTest24(long num) {Test m24Bits = num;}
#undef Test
	
	long get8() {return (m8Bits << 16);}
	long get16() {return (m16Bits << 8);}
	long get24() {return m24Bits;}
	};

class CEasySample {
protected:
	CSample* Data;
public:
	CEasySample() {}
	~CEasySample() {}
	void SetData(CSample* s) {Data = s;}
	virtual long get() = 0;
	virtual void put(long) = 0;
	virtual void putTest(long) = 0;
	virtual void Advance(long) = 0;
	};

#define defEasy(nB, nBits) 																	\
class CEasySample ## nB : public CEasySample{									\
public:																									\
	long get() {return Data->get ## nB();}												\
	void put(long d) {Data->put ## nB(d);}												\
	void putTest(long d) {Data->putTest ## nB(d);}									\
	void Advance(long o) {Data = (CSample*)((long)Data + nBits*o);}	\
	};

defEasy(8,1)
defEasy(16,2)
defEasy(24,3)

#define NewEasy(sound, easy)									\
	switch ((sound)->NBytes()) {									\
		case 1: easy = new CEasySample8(); break;		\
		case 2: easy = new CEasySample16(); break;	\
		case 3: easy = new CEasySample24(); break;	\
		}

class CEvent {
	EventRecord theEvent;
public:
	CEvent(short w=0, long m=0, short mod=0) {theEvent.what = w; 
		theEvent.modifiers = mod; theEvent.message = m;}
	CEvent(EventRecord Ev) {theEvent = Ev;}
	~CEvent() {}
	
	EventRecord GetEvent() {return theEvent;}
	};

class CResFileBase {
public:
	virtual void CloseFile() = 0;
	virtual string GetName() = 0;
	
	virtual void WriteResource(OSType, short, Handle, Str255 = "\p") = 0;
	virtual void DeleteResource(OSType, short) = 0;
	virtual Handle ReadResource(OSType, short) = 0;
	virtual void UseThisFile() = 0;
	virtual void UseHomeFile() = 0;
};

#endif