#ifndef CUTILS
#define CUTILS

#include "CList.h"
#include <Palettes.h>
#include <complex.h>

class CWindowBase;
class CApplicationBase;
class CResFileBase;

struct World {
public:
	World();
	~World();
	GWorldPtr		theWorld;
	GDHandle		theDevice;
};

class CWorldObj {
	CList<World> Worlds;
public:
	void SaveWorld();
	void RestoreWorld();
};

class COffScreenWorld: public CWorldObj{
protected:
	GWorldPtr theWorld;
	PixMapHandle myPixMap;
	CTabHandle myCTable;
	Rect WorldRect;
	short Depth;
	short locked;
	CApplicationBase* theCreator;
	
	void _Construct(short, Rect);
public:
	COffScreenWorld(short, Rect, CApplicationBase*);
	~COffScreenWorld();
	
	void CopyTo(CWindowBase*, Rect&, Rect&);
	void CopyTo(COffScreenWorld*, Rect&, Rect&);
	void MySetPort() {SaveWorld(); SetGWorld(theWorld, 0);}
	
	void LockPixels() {::LockPixels(myPixMap); locked = true;}
	void UnLockPixels() {::UnlockPixels(myPixMap); locked = false;}
	long GetAddress() {return (long)GetPixBaseAddr(myPixMap);}
	long GetRowWidth() {return ((**myPixMap).rowBytes & 0x3FFF);}
	Rect GetRect() {return WorldRect;}
	
	void ChangeRect(Rect);
	void ChangeDepth(short);
	void ChangeWorld(Rect, short);
	void ChangePalette(PaletteHandle);
	};
	
class CPtr {
	Ptr mData;
public:
	CPtr(long = -1, short = false);
	~CPtr() {::DisposePtr(mData);}
	long GetAddress() {return (long)mData;}
	long GetSize() {return GetPtrSize(mData);}
	long SetSize(long newSize) {SetPtrSize(mData, newSize); return GetSize();}
	};
	
class CHandle {
protected:
	Handle mHandle;
public:
	CHandle(long);
	~CHandle();
	
	virtual void SetSize(long);
	virtual long GetSize();
	virtual void *GetDatas(long=0);
	
	virtual Handle GetHandle() {return mHandle;}
	virtual void SetHandle(Handle, short = true);
	friend class CLock;
	};
	
class CIOHandle: public CHandle {
protected:
	long theSize;
	long theMark;
public:
	CIOHandle();
	~CIOHandle();
	
	virtual long GetMark() {return theMark;}
	virtual void SetMark(long m) {theMark = m;}
	virtual void SetSize(long);
	virtual void SetHandle(Handle, short = true);
	
	void WriteData(void*, long);
	void ReadData(void*, long);

	CIOHandle &operator<<(short);
	CIOHandle &operator<<(long);
	CIOHandle &operator<<(double);
	CIOHandle &operator<<(complex<double>);
	CIOHandle &operator<<(Boolean);
	CIOHandle &operator<<(string&);
	
	CIOHandle &operator>>(short&);
	CIOHandle &operator>>(long&);
	CIOHandle &operator>>(double&);
	CIOHandle &operator>>(complex<double>&);
	CIOHandle &operator>>(Boolean&);
	CIOHandle &operator>>(string&);
	};

class CLock {
	CHandle *mHandle;
public:
	CLock(CHandle *theHandle) {mHandle = theHandle; HLock(mHandle->mHandle);}
	~CLock() {HUnlock(mHandle->mHandle);}
	};
	
class CMenu {
	MenuHandle theMenu;
	short menuID;
	CResFileBase* theFile;
public:
	CMenu(MenuHandle, short = -1, CResFileBase* = 0);
	~CMenu() {}
	
	MenuHandle GetMenu() {return theMenu;}
	short GetID() {return menuID;}
	CResFileBase* GetFile() {return theFile;}
	
	short NItems() {return CountMItems(theMenu);}
	void EnDis(short Item, short Enable) {(Enable)?EnableItem(theMenu, Item):DisableItem(theMenu,Item);}
	void SetMark(short Item, short Mark) {SetItemMark(theMenu, Item, Mark);}
	short GetMark(short Item) {short Mark; GetItemMark(theMenu, Item, &Mark); return Mark;}
	};

	
const OSType ApplicationType = 'AMII';

Rect ScreenBits();
	
#define Min(A, B) (((A) < (B)) ? (A) : (B))
#define Max(A, B) (((A) > (B)) ? (A) : (B))
#define pi 3.14159265358979

#define HighWord(aLong)		((aLong) >> 16)
#define LowWord(aLong)		((aLong) & 0x0000FFFF)

#define StrSize(s,T) T = 0;while (s[T]) {T++;} T++
void ShadowRect(Rect, short= true);

#endif